test_schema = {
    'collections': [
        {'name': 'tests_topic', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'question_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_question.id', 'inverse_of': 'topic', 'relationship': 'HasMany',
             'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_question', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'choice_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_choice.id', 'inverse_of': 'question', 'relationship': 'HasMany',
             'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'question_text', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': 'how are you?',
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is longer than', 'message': 'Ensure this value has at least 1 characters', 'value': 1},
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200}]},
            {'field': 'pub_date', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': '2020-05-17 00:00:00', 'integration': None,
             'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'topic', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_topic.id', 'inverse_of': 'question_set', 'relationship': 'BelongsTo',
             'widget': None}]},
        {'name': 'tests_choice', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'question', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_question.id', 'inverse_of': 'choice_set', 'relationship': 'BelongsTo', 'widget': None},
            {'field': 'choice_text', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200},
                 {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'votes', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': 0, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [{'type': 'is greater than',
                                                                                        'message': 'Ensure this value is greater than or equal to -2147483648 characters',
                                                                                        'value': -2147483648},
                                                                                       {'type': 'is less than',
                                                                                        'message': 'Ensure this value is less than or equal to 2147483647 characters',
                                                                                        'value': 2147483647}]}]},
        {'name': 'tests_car', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'wheels', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_wheel.id', 'inverse_of': 'car', 'relationship': 'HasMany', 'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'brand', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_wheel', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'car', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_car.brand', 'inverse_of': 'wheels', 'relationship': 'BelongsTo', 'widget': None}]},
        {'name': 'tests_session', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'session_key', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 40 characters', 'value': 40},
                 {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'session_data', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [{'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'expire_date', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [{'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_publication', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'article_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_article.id', 'inverse_of': 'publications',
             'relationship': 'HasMany', 'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'title', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 30 characters', 'value': 30},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_article_publications', 'is_virtual': False, 'icon': None, 'is_read_only': False,
         'is_searchable': True, 'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None,
         'actions': [], 'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'article', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_article.id', 'inverse_of': 'Article_publications+', 'relationship': 'BelongsTo',
             'widget': None}, {'field': 'publication', 'type': 'Number', 'is_filterable': True, 'is_sortable': True,
                               'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
                               'integration': None, 'reference': 'tests_publication.id',
                               'inverse_of': 'Article_publications+', 'relationship': 'BelongsTo', 'widget': None}]},
        {'name': 'tests_article', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'headline', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 100 characters', 'value': 100},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'publications', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_publication.id', 'inverse_of': 'article_set',
             'relationship': 'HasMany', 'widget': None}]},
        {'name': 'tests_place', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'restaurant', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_restaurant.place_id', 'inverse_of': 'place', 'relationship': 'HasOne', 'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 50 characters', 'value': 50},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'address', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 80 characters', 'value': 80},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_restaurant', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'waiter_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_waiter.id', 'inverse_of': 'restaurant', 'relationship': 'HasMany',
             'widget': None},
            {'field': 'place', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_place.id', 'inverse_of': 'restaurant', 'relationship': 'HasOne', 'widget': None},
            {'field': 'serves_hot_dogs', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': False,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'serves_pizza', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': False,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None}]},
        {'name': 'tests_waiter', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'restaurant', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_restaurant.place_id', 'inverse_of': 'waiter_set', 'relationship': 'BelongsTo',
             'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 50 characters', 'value': 50},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_student', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'year_in_school', 'type': 'Enum', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': 'FR',
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 2 characters', 'value': 2}],
             'enums': ['FR', 'SO', 'JR', 'SR', 'GR']}]},
        {'name': 'tests_chessboard', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'board', 'type': ['String'], 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None}]},
        {'name': 'tests_serial', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'uuid', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None}]},
        {'name': 'auth_permission', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'group_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_group.id', 'inverse_of': 'permissions', 'relationship': 'HasMany',
             'widget': None}, {'field': 'user_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
                               'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
                               'integration': None, 'reference': 'auth_user.id', 'inverse_of': 'user_permissions',
                               'relationship': 'HasMany', 'widget': None},
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 255 characters', 'value': 255},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'content_type', 'type': 'Number', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'django_content_type.id', 'inverse_of': 'permission_set',
             'relationship': 'BelongsTo', 'widget': None},
            {'field': 'codename', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 100 characters', 'value': 100},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'auth_group_permissions', 'is_virtual': False, 'icon': None, 'is_read_only': False,
         'is_searchable': True, 'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None,
         'actions': [], 'segments': [], 'fields': [
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'group', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_group.id', 'inverse_of': 'Group_permissions+', 'relationship': 'BelongsTo',
             'widget': None},
            {'field': 'permission', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_permission.id', 'inverse_of': 'Group_permissions+', 'relationship': 'BelongsTo',
             'widget': None}]},
        {'name': 'auth_group', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'user_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_user.id', 'inverse_of': 'groups', 'relationship': 'HasMany',
             'widget': None},
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 150 characters', 'value': 150},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'permissions', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_permission.id', 'inverse_of': 'group_set',
             'relationship': 'HasMany', 'widget': None}]},
        {'name': 'auth_user_groups', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'user', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_user.id', 'inverse_of': 'User_groups+', 'relationship': 'BelongsTo', 'widget': None},
            {'field': 'group', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_group.id', 'inverse_of': 'User_groups+', 'relationship': 'BelongsTo', 'widget': None}]},
        {'name': 'auth_user_user_permissions', 'is_virtual': False, 'icon': None, 'is_read_only': False,
         'is_searchable': True, 'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None,
         'actions': [], 'segments': [], 'fields': [
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'user', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_user.id', 'inverse_of': 'User_user_permissions+', 'relationship': 'BelongsTo',
             'widget': None},
            {'field': 'permission', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_permission.id', 'inverse_of': 'User_user_permissions+', 'relationship': 'BelongsTo',
             'widget': None}]},
        {'name': 'auth_user', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'password', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 128 characters', 'value': 128},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'last_login', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'is_superuser', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': False,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'username', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is like', 'message': 'Ensure this value match your pattern', 'value': '^[\\w.@+-]+\\Z'},
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 150 characters', 'value': 150},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'first_name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 150 characters', 'value': 150},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'last_name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 150 characters', 'value': 150},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'email', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 254 characters', 'value': 254},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'is_staff', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': False, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'is_active', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': True, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'date_joined', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'groups', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_group.id', 'inverse_of': 'user_set', 'relationship': 'HasMany', 'widget': None},
            {'field': 'user_permissions', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_permission.id', 'inverse_of': 'user_set',
             'relationship': 'HasMany', 'widget': None}]},
        {'name': 'django_content_type', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'permission_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_permission.id', 'inverse_of': 'content_type',
             'relationship': 'HasMany', 'widget': None},
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'app_label', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 100 characters', 'value': 100},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'model', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 100 characters', 'value': 100},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]}],
    'meta': {'liana': 'django-forestadmin', 'liana_version': '0.0.0',
             'stack': {'database_type': 'sqlite', 'orm_version': '9.9.9'}}}

test_choice_schema = {
    'collections': [
        {'name': 'tests_choice', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'choice_text', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200},
                 {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'votes', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': 0, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [{'type': 'is greater than',
                                                                                        'message': 'Ensure this value is greater than or equal to -2147483648 characters',
                                                                                        'value': -2147483648},
                                                                                       {'type': 'is less than',
                                                                                        'message': 'Ensure this value is less than or equal to 2147483647 characters',
                                                                                        'value': 2147483647}]}]}],
    'meta': {'liana': 'django-forestadmin', 'liana_version': '0.0.0',
             'stack': {'database_type': 'sqlite', 'orm_version': '9.9.9'}}}

test_question_schema_data = {'collections': [
    {
        'name': 'tests_question', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
        'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None,
        'actions': [
            {'name': 'Send invoice', 'type': 'single', 'fields': [
                {'field': 'country', 'type': 'Enum', 'enums': ['FR', 'US'], 'isReadOnly': False, 'isRequired': False,
                 'defaultValue': None, 'integration': None, 'reference': None, 'description': None, 'widget': None,
                 'position': 0},
                {'field': 'city', 'type': 'String', 'isReadOnly': False, 'isRequired': False, 'defaultValue': None,
                 'integration': None, 'reference': None, 'description': None, 'widget': None,
                 'hook': 'cityChange',
                 'position': 1},
                {'field': 'zip code', 'type': 'String', 'is_read_only': False, 'is_required': False,
                 'default_value': None,
                 'integration': None, 'reference': None, 'description': None, 'widget': None,
                 'hook': 'zipCodeChange',
                 'position': 2}], 'hooks': {'load': True, 'change': ['cityChange', 'zipCodeChange']},
             'endpoint': '/forest/actions/send-invoice', 'baseUrl': None, 'httpMethod': 'POST', 'redirect': None,
             'download': False}],
        'segments': [{'name': 'Best questions'}],
        'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'question_text', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is longer than', 'message': 'Ensure this value has at least 1 characters', 'value': 1},
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'pub_date', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'foo', 'is_virtual': True, 'type': '', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'bar', 'is_virtual': True, 'type': '', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None}]}],
    'meta': {
        'liana': 'django-forestadmin',
        'liana_version': '0.0.0',
        'stack': {
            'database_type': 'sqlite',
            'orm_version': '9.9.9'
        },
    }}

test_exclude_django_contrib_schema = {
    'collections': [
        {'name': 'tests_topic', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'question_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_question.id', 'inverse_of': 'topic', 'relationship': 'HasMany',
             'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_question', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'choice_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_choice.id', 'inverse_of': 'question', 'relationship': 'HasMany',
             'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'question_text', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': 'how are you?',
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is longer than', 'message': 'Ensure this value has at least 1 characters', 'value': 1},
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200}]},
            {'field': 'pub_date', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': '2020-05-17 00:00:00', 'integration': None,
             'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'topic', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_topic.id', 'inverse_of': 'question_set', 'relationship': 'BelongsTo',
             'widget': None}]},
        {'name': 'tests_choice', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'question', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_question.id', 'inverse_of': 'choice_set', 'relationship': 'BelongsTo', 'widget': None},
            {'field': 'choice_text', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200},
                 {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'votes', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': 0, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [{'type': 'is greater than',
                                                                                        'message': 'Ensure this value is greater than or equal to -2147483648 characters',
                                                                                        'value': -2147483648},
                                                                                       {'type': 'is less than',
                                                                                        'message': 'Ensure this value is less than or equal to 2147483647 characters',
                                                                                        'value': 2147483647}]}]},
        {'name': 'tests_car', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'wheels', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_wheel.id', 'inverse_of': 'car', 'relationship': 'HasMany', 'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'brand', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters', 'value': 200},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_wheel', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'car', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_car.brand', 'inverse_of': 'wheels', 'relationship': 'BelongsTo', 'widget': None}]},
        {'name': 'tests_session', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'session_key', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 40 characters', 'value': 40},
                 {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'session_data', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [{'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'expire_date', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [{'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_publication', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'article_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_article.id', 'inverse_of': 'publications',
             'relationship': 'HasMany', 'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'title', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 30 characters', 'value': 30},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_article_publications', 'is_virtual': False, 'icon': None, 'is_read_only': False,
         'is_searchable': True, 'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None,
         'actions': [], 'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'article', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_article.id', 'inverse_of': 'Article_publications+', 'relationship': 'BelongsTo',
             'widget': None}, {'field': 'publication', 'type': 'Number', 'is_filterable': True, 'is_sortable': True,
                               'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
                               'integration': None, 'reference': 'tests_publication.id',
                               'inverse_of': 'Article_publications+', 'relationship': 'BelongsTo', 'widget': None}]},
        {'name': 'tests_article', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'headline', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 100 characters', 'value': 100},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'publications', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_publication.id', 'inverse_of': 'article_set',
             'relationship': 'HasMany', 'widget': None}]},
        {'name': 'tests_place', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'restaurant', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_restaurant.place_id', 'inverse_of': 'place', 'relationship': 'HasOne', 'widget': None},
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 50 characters', 'value': 50},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'address', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 80 characters', 'value': 80},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_restaurant', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'waiter_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'tests_waiter.id', 'inverse_of': 'restaurant', 'relationship': 'HasMany',
             'widget': None},
            {'field': 'place', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_place.id', 'inverse_of': 'restaurant', 'relationship': 'HasOne', 'widget': None},
            {'field': 'serves_hot_dogs', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': False,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'serves_pizza', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': False,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None}]},
        {'name': 'tests_waiter', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'restaurant', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'tests_restaurant.place_id', 'inverse_of': 'waiter_set', 'relationship': 'BelongsTo',
             'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 50 characters', 'value': 50},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'tests_student', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'year_in_school', 'type': 'Enum', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': 'FR',
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
             'validations': [
                 {'type': 'is shorter than', 'message': 'Ensure this value has at most 2 characters', 'value': 2}],
             'enums': ['FR', 'SO', 'JR', 'SR', 'GR']}]},
        {'name': 'tests_chessboard', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'board', 'type': ['String'], 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None}]},
        {'name': 'tests_serial', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'uuid', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None}]},
        {'name': 'auth_permission', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'group_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_group.id', 'inverse_of': 'permissions', 'relationship': 'HasMany',
             'widget': None}, {'field': 'user_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
                               'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
                               'integration': None, 'reference': 'auth_user.id', 'inverse_of': 'user_permissions',
                               'relationship': 'HasMany', 'widget': None},
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 255 characters', 'value': 255},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'content_type', 'type': 'Number', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'django_content_type.id', 'inverse_of': 'permission_set',
             'relationship': 'BelongsTo', 'widget': None},
            {'field': 'codename', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 100 characters', 'value': 100},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]},
        {'name': 'auth_group_permissions', 'is_virtual': False, 'icon': None, 'is_read_only': False,
         'is_searchable': True, 'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None,
         'actions': [], 'segments': [], 'fields': [
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'group', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_group.id', 'inverse_of': 'Group_permissions+', 'relationship': 'BelongsTo',
             'widget': None},
            {'field': 'permission', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_permission.id', 'inverse_of': 'Group_permissions+', 'relationship': 'BelongsTo',
             'widget': None}]},
        {'name': 'auth_group', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'user_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_user.id', 'inverse_of': 'groups', 'relationship': 'HasMany',
             'widget': None},
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 150 characters', 'value': 150},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'permissions', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_permission.id', 'inverse_of': 'group_set',
             'relationship': 'HasMany', 'widget': None}]},
        {'name': 'auth_user_groups', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'user', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_user.id', 'inverse_of': 'User_groups+', 'relationship': 'BelongsTo', 'widget': None},
            {'field': 'group', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_group.id', 'inverse_of': 'User_groups+', 'relationship': 'BelongsTo', 'widget': None}]},
        {'name': 'auth_user_user_permissions', 'is_virtual': False, 'icon': None, 'is_read_only': False,
         'is_searchable': True, 'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None,
         'actions': [], 'segments': [], 'fields': [
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'user', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_user.id', 'inverse_of': 'User_user_permissions+', 'relationship': 'BelongsTo',
             'widget': None},
            {'field': 'permission', 'type': 'Number', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_permission.id', 'inverse_of': 'User_user_permissions+', 'relationship': 'BelongsTo',
             'widget': None}]},
        {'name': 'auth_user', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'password', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 128 characters', 'value': 128},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'last_login', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'is_superuser', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': False,
             'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'username', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is like', 'message': 'Ensure this value match your pattern', 'value': '^[\\w.@+-]+\\Z'},
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 150 characters', 'value': 150},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'first_name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 150 characters', 'value': 150},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'last_name', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 150 characters', 'value': 150},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'email', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 254 characters', 'value': 254},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'is_staff', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': False, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'is_active', 'type': 'Boolean', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': True, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'date_joined', 'type': 'Date', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'groups', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None,
             'reference': 'auth_group.id', 'inverse_of': 'user_set', 'relationship': 'HasMany', 'widget': None},
            {'field': 'user_permissions', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_permission.id', 'inverse_of': 'user_set',
             'relationship': 'HasMany', 'widget': None}]},
        {'name': 'django_content_type', 'is_virtual': False, 'icon': None, 'is_read_only': False, 'is_searchable': True,
         'only_for_relationships': False, 'pagination_type': 'page', 'search_fields': None, 'actions': [],
         'segments': [], 'fields': [
            {'field': 'permission_set', 'type': ['Number'], 'is_filterable': False, 'is_sortable': True,
             'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
             'integration': None, 'reference': 'auth_permission.id', 'inverse_of': 'content_type',
             'relationship': 'HasMany', 'widget': None},
            {'field': 'id', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None},
            {'field': 'app_label', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 100 characters', 'value': 100},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
            {'field': 'model', 'type': 'String', 'is_filterable': True, 'is_sortable': True, 'is_read_only': False,
             'is_required': False, 'is_virtual': False, 'default_value': None, 'integration': None, 'reference': None,
             'inverse_of': None, 'relationship': None, 'widget': None, 'validations': [
                {'type': 'is shorter than', 'message': 'Ensure this value has at most 100 characters', 'value': 100},
                {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]}]}],
    'meta': {'liana': 'django-forestadmin', 'liana_version': '0.0.0',
             'stack': {'database_type': 'sqlite', 'orm_version': '9.9.9'}}}

test_serialized_schema = {
    'data': [{'id': 'tests_question', 'type': 'collections',
              'attributes': {'name': 'tests_question', 'is_virtual': False, 'icon': None, 'is_read_only': False,
                             'is_searchable': True, 'only_for_relationships': False, 'pagination_type': 'page',
                             'search_fields': None, 'actions': [{'name': 'Send invoice', 'type': 'single', 'fields': [
                      {'field': 'country', 'type': 'Enum', 'enums': ['FR', 'US'], 'isReadOnly': False,
                       'isRequired': False, 'defaultValue': None, 'integration': None, 'reference': None,
                       'description': None, 'widget': None, 'position': 0},
                      {'field': 'city', 'type': 'String', 'isReadOnly': False, 'isRequired': False,
                       'defaultValue': None, 'integration': None, 'reference': None, 'description': None,
                       'widget': None, 'hook': 'cityChange', 'position': 1},
                      {'field': 'zip code', 'type': 'String', 'is_read_only': False, 'is_required': False,
                       'default_value': None, 'integration': None, 'reference': None, 'description': None,
                       'widget': None, 'hook': 'zipCodeChange', 'position': 2}], 'hooks': {'load': True,
                                                                                           'change': ['cityChange',
                                                                                                      'zipCodeChange']},
                                                                 'endpoint': '/forest/actions/send-invoice',
                                                                 'baseUrl': None, 'httpMethod': 'POST',
                                                                 'redirect': None, 'download': False}],
                             'segments': [{'name': 'Best questions'}], 'fields': [
                      {'field': 'id', 'type': 'Number', 'is_filterable': True, 'is_sortable': True,
                       'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
                       'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None,
                       'widget': None},
                      {'field': 'question_text', 'type': 'String', 'is_filterable': True, 'is_sortable': True,
                       'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
                       'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None,
                       'validations': [
                           {'type': 'is longer than', 'message': 'Ensure this value has at least 1 characters',
                            'value': 1},
                           {'type': 'is shorter than', 'message': 'Ensure this value has at most 200 characters',
                            'value': 200},
                           {'type': 'is present', 'message': 'Ensure this value is not null or not empty'}]},
                      {'field': 'pub_date', 'type': 'Date', 'is_filterable': True, 'is_sortable': True,
                       'is_read_only': False, 'is_required': False, 'is_virtual': False, 'default_value': None,
                       'integration': None, 'reference': None, 'inverse_of': None, 'relationship': None,
                       'widget': None},
                      {'field': 'foo', 'is_virtual': True, 'type': '', 'is_filterable': True, 'is_sortable': True,
                       'is_read_only': False, 'is_required': False, 'default_value': None, 'integration': None,
                       'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None},
                      {'field': 'bar', 'is_virtual': True, 'type': '', 'is_filterable': True, 'is_sortable': True,
                       'is_read_only': False, 'is_required': False, 'default_value': None, 'integration': None,
                       'reference': None, 'inverse_of': None, 'relationship': None, 'widget': None}]},
              'relationships': {'actions': {'data': [{'id': 'tests_question.Send invoice', 'type': 'actions'}]},
                                'segments': {'data': [{'id': 'tests_question.Best questions', 'type': 'segments'}]}}}],
    'included': [{'id': 'tests_question.Send invoice', 'type': 'actions',
                  'attributes': {'name': 'Send invoice', 'type': 'single', 'fields': [
                      {'field': 'country', 'type': 'Enum', 'enums': ['FR', 'US'], 'isReadOnly': False,
                       'isRequired': False, 'defaultValue': None, 'integration': None, 'reference': None,
                       'description': None, 'widget': None, 'position': 0},
                      {'field': 'city', 'type': 'String', 'isReadOnly': False, 'isRequired': False,
                       'defaultValue': None, 'integration': None, 'reference': None, 'description': None,
                       'widget': None, 'hook': 'cityChange', 'position': 1},
                      {'field': 'zip code', 'type': 'String', 'is_read_only': False, 'is_required': False,
                       'default_value': None, 'integration': None, 'reference': None, 'description': None,
                       'widget': None, 'hook': 'zipCodeChange', 'position': 2}],
                                 'hooks': {'load': True, 'change': ['cityChange', 'zipCodeChange']},
                                 'endpoint': '/forest/actions/send-invoice', 'baseUrl': None, 'httpMethod': 'POST',
                                 'redirect': None, 'download': False}},
                 {'id': 'tests_question.Best questions', 'type': 'segments', 'attributes': {'name': 'Best questions'}}],
    'meta': {'liana': 'django-forestadmin', 'liana_version': '0.0.0',
             'stack': {'database_type': 'sqlite', 'orm_version': '9.9.9'}}}



import copy
from unittest import mock

from django.conf import settings
from django.test import TestCase
from django.urls import reverse

from django_forest.tests.fixtures.schema import test_schema
from django_forest.tests.middleware import mocked_ip_whitelist, mocked_config, \
    mocked_requests_permission
from django_forest.utils.ip_whitelist import IpWhitelist
from django_forest.utils.middlewares import set_middlewares
from django_forest.utils.permissions import Permission
from django_forest.utils.schema import Schema
from django_forest.utils.schema.json_api_schema import JsonApiSchema
from django_forest.utils.scope import ScopeManager


mocked_ip_whitelist_ip = copy.deepcopy(mocked_ip_whitelist)
mocked_ip_whitelist_ip['data']['attributes']['use_ip_whitelist'] = True
mocked_ip_whitelist_ip['data']['attributes']['rules'] = [
    {
        'type': 0,
        'ip': '123.12.34.0'
    }
]
mocked_ip_whitelist_ip_loopback = copy.deepcopy(mocked_ip_whitelist)
mocked_ip_whitelist_ip_loopback['data']['attributes']['use_ip_whitelist'] = True
mocked_ip_whitelist_ip_loopback['data']['attributes']['rules'] = [
    {
        'type': 0,
        'ip': '127.0.0.1'
    }
]
mocked_ip_whitelist_range = copy.deepcopy(mocked_ip_whitelist)
mocked_ip_whitelist_range['data']['attributes']['use_ip_whitelist'] = True
mocked_ip_whitelist_range['data']['attributes']['rules'] = [
    {
        'type': 1,
        'ipMinimum': '10.0.0.1',
        'ipMaximum': '200.0.0.1'
    }
]
mocked_ip_whitelist_subnet = copy.deepcopy(mocked_ip_whitelist)
mocked_ip_whitelist_subnet['data']['attributes']['use_ip_whitelist'] = True
mocked_ip_whitelist_subnet['data']['attributes']['rules'] = [
    {
        'type': 2,
        'range': '123.12.34.0/31'
    }
]


mocked_config_server_error = copy.deepcopy(mocked_config)
mocked_config_server_error['ip_whitelist']['status'] = 400
mocked_config_ip = copy.deepcopy(mocked_config)
mocked_config_ip['ip_whitelist']['data'] = mocked_ip_whitelist_ip
mocked_config_ip_loopback = copy.deepcopy(mocked_config)
mocked_config_ip_loopback['ip_whitelist']['data'] = mocked_ip_whitelist_ip_loopback
mocked_config_range = copy.deepcopy(mocked_config)
mocked_config_range['ip_whitelist']['data'] = mocked_ip_whitelist_range
mocked_config_subnet = copy.deepcopy(mocked_config)
mocked_config_subnet['ip_whitelist']['data'] = mocked_ip_whitelist_subnet


@mock.patch('jose.jwt.decode', return_value={'id': 1, 'rendering_id': 1})
class MiddlewareIpWhitelistTests(TestCase):

    def setUp(self):
        set_middlewares()
        Schema.schema = copy.deepcopy(test_schema)
        Schema.handle_json_api_schema()
        self.url = reverse('django_forest:resources:list', kwargs={'resource': 'tests_question'})
        self.client = self.client_class(
            HTTP_AUTHORIZATION='Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjUiLCJlbWFpbCI6Imd1aWxsYXVtZWNAZm9yZXN0YWRtaW4uY29tIiwiZmlyc3RfbmFtZSI6Ikd1aWxsYXVtZSIsImxhc3RfbmFtZSI6IkNpc2NvIiwidGVhbSI6Ik9wZXJhdGlvbnMiLCJyZW5kZXJpbmdfaWQiOjEsImV4cCI6MTYyNTY3OTYyNi44ODYwMTh9.mHjA05yvMr99gFMuFv0SnPDCeOd2ZyMSN868V7lsjnw')

    def tearDown(self):
        # reset _registry after each test
        JsonApiSchema._registry = {}
        Permission.permissions_cached = {}
        Permission.renderings_cached = {}
        ScopeManager.cache = {}
        IpWhitelist.fetched = False
        settings.MIDDLEWARE.remove('django_forest.middleware.PermissionMiddleware')
        settings.MIDDLEWARE.remove('django_forest.middleware.IpWhitelistMiddleware')

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_server_error))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='123.12.34.0')
    def test_server_error(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='123.12.34.0')
    def test_no_rules(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_ip))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value=None)
    def test_no_ip(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_ip))
    def test_machine_ip(self, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_ip))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='123.12.34.0')
    def test_ip(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_ip))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='123.12.34.1')
    def test_ip_invalid(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_ip))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='2001:db8::1000')
    def test_ip_v6_invalid(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_ip_loopback))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='127.0.0.2')
    def test_ip_loopback(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_range))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='123.12.34.5')
    def test_range(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_range))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='223.12.34.5')
    def test_range_invalid(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_range))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='2001:db8::1000')
    def test_range_ipv6_invalid(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_subnet))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='123.12.34.1')
    def test_no_subnet(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url, {
            'page[number]': '1',
            'page[size]': '15'
        })
        self.assertEqual(response.status_code, 200)

    @mock.patch('requests.get', side_effect=mocked_requests_permission(mocked_config_subnet))
    @mock.patch('django_forest.middleware.ip_whitelist.IpWhitelistMiddleware.get_client_ip', return_value='123.12.34.3')
    def test_subnet_invalid(self, mocked_ip, mocked_requests, mocked_decode):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)



import pytest
import pandas
from unittest import mock
from datetime import datetime
from typing import Tuple
try:
    import zoneinfo
except ImportError:
    from backports import zoneinfo

from django_forest.resources.utils.queryset.filters.date.utils import (
    get_previous_x_days,
    get_next_x_days,
    get_previous_x_weeks,
    get_previous_x_months,
    get_previous_x_years,
    get_previous_x_quarters,
    get_date_range,
    get_previous_x_hours,
    get_next_x_hours,
)

@pytest.mark.parametrize(
    "current,days,expected,offset",
    (
        (
            datetime(2021, 1, 12, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            1, 
            (
                datetime(2021, 1, 11, 0, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2021, 1, 12, 0, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            0
        ),
        (
            datetime(2021, 1, 12, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            1, 
            (
                datetime(2021, 1, 10, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2021, 1, 11, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            1
        ),
        (
            datetime(2021, 1, 12, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            2, 
            (
                datetime(2021, 1, 10, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 1, 12, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            0
        ),
        (
            datetime(2021, 1, 12, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            2, 
            (
                datetime(2021, 1, 8, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 1, 10, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            1
        ),
        (
            datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            3, 
            (
                datetime(2020, 12, 29, tzinfo=zoneinfo.ZoneInfo("UTC")), 
                datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            0
        ),
        (
            datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            3, 
            (
                datetime(2020, 12, 26, tzinfo=zoneinfo.ZoneInfo("UTC")), 
                datetime(2020, 12, 29, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            1
        ),
        (
            datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            3, 
            (
                datetime(2020, 12, 23, tzinfo=zoneinfo.ZoneInfo("UTC")), 
                datetime(2020, 12, 26, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            2
        ),
        (
            datetime(2022, 3, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
            1, 
            (
                datetime(2022, 2, 28, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
                datetime(2022, 3, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            ),
            0
        ),
        (
            datetime(2022, 3, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
            365, 
            (
                datetime(2021, 3, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
                datetime(2022, 3, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo"))
            ),
            0
        ),
        (
            datetime(2022, 3, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
            365, 
            (
                datetime(2011, 3, 4, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
                datetime(2012, 3, 3, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo"))
            ),
            10
        ),
        (
            datetime(2024, 3, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")), 
            1, 
            (
                datetime(2024, 2, 29, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")), 
                datetime(2024, 3, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris"))
            ),
            0
        ),
        (
            datetime(2024, 3, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Chungking")), 
            365, 
            (
                datetime(2023, 3, 2, tzinfo=zoneinfo.ZoneInfo("Asia/Chungking")), 
                datetime(2024, 3, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Chungking"))
            ),
            0
        ),
    )
)
def test_get_previous_x_days(current: datetime, days: int, expected: Tuple[datetime], offset: int):
    assert get_previous_x_days(current, days, offset) == expected


@pytest.mark.parametrize(
    "current,days,expected,offset",
    (
        (
            datetime(2021, 1, 12, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            1, 
            (
                datetime(2021, 1, 12, 0, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2021, 1, 13, 0, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            0
        ),
        (
            datetime(2021, 1, 12, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            1, 
            (
                datetime(2021, 1, 13, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2021, 1, 14, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            1
        ),
            (
            datetime(2021, 1, 12, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            2, 
            (
                datetime(2021, 1, 12, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 1, 14, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            0
        ),
        (
            datetime(2021, 1, 12, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            2, 
            (
                datetime(2021, 1, 18, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 1, 20, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            3
        ),
        (
            datetime(2020, 12, 29, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            3, 
            (
                datetime(2020, 12, 29, tzinfo=zoneinfo.ZoneInfo("UTC")), 
                datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            0
        ),
        (
            datetime(2020, 12, 29, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            3, 
            (
                datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC")), 
                datetime(2021, 1, 4, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            1
        ),
        (
            datetime(2022, 2, 28, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
            1, 
            (
                datetime(2022, 2, 28, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
                datetime(2022, 3, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            ),
            0
        ),
        (
            datetime(2022, 2, 28, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
            365, 
            (
                datetime(2022, 2, 28, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
                datetime(2023, 2, 28, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo"))
            ),
            0
        ),
        (
            datetime(2022, 2, 28, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
            365, 
            (
                datetime(2023, 2, 28, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
                datetime(2024, 2, 28, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo"))
            ),
            1
        ),
        (
            datetime(2024, 2, 29, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")), 
            1, 
            (
                datetime(2024, 2, 29, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")), 
                datetime(2024, 3, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris"))
            ),
            0
        ),
    )
)
def test_get_next_x_days(current: datetime, days: int, expected: Tuple[datetime], offset: int):
    assert get_next_x_days(current, days, offset) == expected

@pytest.mark.parametrize(
    "current,weeks,expected,offset",
    (
        (
            datetime(2022, 1, 5, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            52, 
            (
                datetime(2021, 1, 4, 0, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2022, 1, 3, 0, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            0
        ),
        (
            datetime(2022, 1, 5, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            52, 
            (
                datetime(2020, 1, 6, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2021, 1, 4, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            1
        ),
        (
            datetime(2022, 1, 5, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            1, 
            (
                datetime(2021, 12, 27, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2022, 1, 3, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            0
        ),
        (
            datetime(2022, 1, 5, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            1, 
            (
                datetime(2021, 12, 20, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 12, 27, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            1
        ),
        (
            datetime(2017, 11, 6, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            3, 
            (
                datetime(2017, 10, 16, tzinfo=zoneinfo.ZoneInfo("UTC")),
                datetime(2017, 11, 6, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            0
        ),
        (
            datetime(2017, 11, 6, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            3, 
            (
                datetime(2017, 9, 4, tzinfo=zoneinfo.ZoneInfo("UTC")),
                datetime(2017, 9, 25, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            2
        ),
    )
)
def test_get_previous_x_weeks(current: datetime, weeks: int, expected: Tuple[datetime], offset: int):
    assert get_previous_x_weeks(current, weeks, offset) == expected


@pytest.mark.parametrize(
    "current,months,expected,offset",
    (
        (
            datetime(2022, 1, 4, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            1, 
            (
                datetime(2021, 12, 1, 0, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2022, 1, 1, 0, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            0
        ),
        (
            datetime(2022, 1, 4, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            1, 
            (
                datetime(2021, 11, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2021, 12, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            1
        ),
        (
            datetime(2022, 2, 4, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            1, 
            (
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2022, 2, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            0
        ),
        (
            datetime(2022, 1, 5, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            12, 
            (
                datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            0
        ),
        (
            datetime(2022, 1, 5, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            12, 
            (
                datetime(2018, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC")), 
                datetime(2019, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            3
        ),
        (
            datetime(2022, 1, 5, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
            17, 
            (
                datetime(2020, 8, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            ),
            0
        ),
        (
            datetime(2022, 1, 5, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
            29, 
            (
                datetime(2019, 8, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo"))
            ),
            0
        ),
        (
            datetime(2022, 12, 5, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")), 
            121, 
            (
                datetime(2012, 11, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")), 
                datetime(2022, 12, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris"))
            ),
            0
        ),
    )
)
def test_get_previous_x_months(current: datetime, months: int, expected: Tuple[datetime], offset: int):
    assert get_previous_x_months(current, months, offset) == expected

@pytest.mark.parametrize(
    "current,years,expected,offset",
    (   
        (
            datetime(2022, 1, 12, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            1, 
            (
                datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            0
        ),
        (
            datetime(2022, 1, 12, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            1, 
            (
                datetime(2017, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2018, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            4
        ),
        (
            datetime(2022, 2, 5, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            2, 
            (
                datetime(2020, 1, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            0
        ),
        (
            datetime(2022, 2, 5, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
            2, 
            (
                datetime(2018, 1, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2020, 1, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            1
        ),
        (
            datetime(2022, 3, 31, tzinfo=zoneinfo.ZoneInfo("UTC")), 
            10, 
            (
                datetime(2012, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            0
        ),
        (
            datetime(2022, 4, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
            104, 
            (
                datetime(1918, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            ),
            0
        ),

        (
            datetime(2022, 4, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
            104, 
            (
                datetime(1814, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
                datetime(1918, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            ),
            1
        ),
    )
)
def test_get_previous_x_years(current: datetime, years: int, expected: Tuple[datetime], offset: int):
    assert get_previous_x_years(current, years, offset) == expected

@pytest.mark.parametrize(
    "current,quarter,expected,offset",
    (   
        (	
            datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")),
            1,
            (
                datetime(2021, 10, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            0
        ),
        (	
            datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")),
            1,
            (
                datetime(2021, 4, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2021, 7, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            2
        ),
        (
            datetime(2021, 4, 12, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")),
            1, 
            (
                datetime(2021, 1, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 4, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            0
        ),
        (
            datetime(2012, 3, 31, tzinfo=zoneinfo.ZoneInfo("UTC")),
            2,
            (
                datetime(2011, 7, 1, tzinfo=zoneinfo.ZoneInfo("UTC")),
                datetime(2012, 1, 1, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            0
        ),
        (
            datetime(2018, 12, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
            4, 
            (
                datetime(2017, 10, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
                datetime(2018, 10, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            ),
            0
        ),
        (
            datetime(2018, 12, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
            4, 
            (
                datetime(2014, 10, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")), 
                datetime(2015, 10, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            ),
            3
        ),
        (
            datetime(2018, 12, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")),
            4,
            (
                datetime(2017, 10, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo")),
                datetime(2018, 10, 1, tzinfo=zoneinfo.ZoneInfo("Africa/Malabo"))
            ),
            0
        ),
        (
            datetime(2018, 1, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")),
            3,
            (
                datetime(2017, 4, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")),
                datetime(2018, 1, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris"))
            ),
            0
        ),
    )
)
def test_get_first_day_of_previous_x_quarter(current: datetime, quarter: int, expected: Tuple[datetime], offset: int):
    assert get_previous_x_quarters(current, quarter, offset) == expected

@pytest.mark.parametrize(
    "current,period,expected,offset",
    (   
        (	
            datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")),
            1,
            (
                datetime(2021, 12, 31, 23, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            0
        ),
        (	
            datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")),
            1,
            (
                datetime(2021, 12, 31, 21, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
                datetime(2021, 12, 31, 22, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe"))
            ),
            2
        ),
        (
            datetime(2021, 4, 12, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")),
            24, 
            (
                datetime(2021, 4, 11, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 4, 12, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            0
        ),
        (
            datetime(2021, 4, 12, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")),
            24, 
            (
                datetime(2021, 4, 7, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 4, 8, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            4
        ),
        (
            datetime(2012, 3, 31, 12, tzinfo=zoneinfo.ZoneInfo("UTC")),
            2,
            (
                datetime(2012, 3, 31, 10, tzinfo=zoneinfo.ZoneInfo("UTC")),
                datetime(2012, 3, 31, 12, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            0
        ),
    )
)
def test_get_previous_x_hours(current: datetime, period: int, expected: Tuple[datetime], offset: int):
    assert get_previous_x_hours(current, period, offset) == expected

@pytest.mark.parametrize(
    "current,period,expected,offset",
    (   
        (	
            datetime(2021, 12, 31, 23, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")),
            1,
            (
                datetime(2021, 12, 31, 23, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")),
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            ),
            0
        ),
        (	
            datetime(2021, 12, 31, 23, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")),
            1,
            (
                datetime(2022, 1, 1, 1, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")),
                datetime(2022, 1, 1, 2, tzinfo=zoneinfo.ZoneInfo("Asia/Aqtobe")), 
            ),
            2
        ),
        (
            datetime(2021, 4, 12, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")),
            24, 
            (
                datetime(2021, 4, 12, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 4, 13, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            0
        ),
        (
            datetime(2021, 4, 12, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")),
            24, 
            (
                datetime(2021, 4, 15, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo")), 
                datetime(2021, 4, 16, 1, tzinfo=zoneinfo.ZoneInfo("Antarctica/McMurdo"))
            ),
            3
        ),
        (
            datetime(2012, 3, 31, 10, tzinfo=zoneinfo.ZoneInfo("UTC")),
            2,
            (
                datetime(2012, 3, 31, 10, tzinfo=zoneinfo.ZoneInfo("UTC")),
                datetime(2012, 3, 31, 12, tzinfo=zoneinfo.ZoneInfo("UTC"))
            ),
            0
        ),
    )
)
def test_get_next_x_hours(current: datetime, period: int, expected: Tuple[datetime], offset: int):
    assert get_next_x_hours(current, period, offset) == expected
    
@pytest.mark.parametrize(
    "current,frequency,period,previous,offset,expected",
    ( 
        (
            datetime(2018, 12, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
            'X', 
            2, 
            False, 
            0,
            (
                datetime(2012, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
                datetime(2002, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            )
        ),
        (
            datetime(2018, 12, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
            'XY', 
            2, 
            True, 
            1,
            (
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
                datetime(2012, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            )
        ),
        (
            datetime(2018, 12, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
            'XY', 
            2, 
            True, 
            2,
            (
                datetime(2022, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina")),
                datetime(2012, 1, 1, tzinfo=zoneinfo.ZoneInfo("America/Araguaina"))
            )
        ),
    )
)
def test_get_date_range(current: datetime, frequency: str, period: int, previous: bool, offset: int, expected: Tuple[datetime]):
    with mock.patch('django_forest.resources.utils.queryset.filters.date.utils.pd') as pd:
        pd.date_range.return_value = [
            pandas.Timestamp(2022, 1, 1),
            pandas.Timestamp(2012, 1, 1),
            pandas.Timestamp(2002, 1, 1)
        ]
        pd.to_datetime.return_value = 'faketimestamp'
        res = get_date_range(current, frequency, period, previous, offset)
        pd.to_datetime.assert_called_with(current.replace(tzinfo=zoneinfo.ZoneInfo('UTC')))
        kwargs = {
            'freq': frequency,
            'periods': offset + 2,
        }
        if period:
            kwargs['freq'] = f"{period}{frequency}"
        if previous:
            kwargs["end"] = "faketimestamp"
        else:
            kwargs["start"] = "faketimestamp"

        pd.date_range.assert_called_with(**kwargs)
        assert res == expected


from unittest import mock

from django.test import TestCase
from oic.oic import Client

from django_forest.tests.authentication.oidc.test_configuration_retriever import mocked_config
from django_forest.tests.authentication.oidc.test_dynamic_client_registrator import mocked_client_credentials
from django_forest.authentication.oidc.client_manager import OidcClientManager


class AuthenticationOidcClientManagerTests(TestCase):

    def setUp(self):
        self.callback_url = 'http://localhost:8000/authentication/callback'

    def tearDown(self):
        OidcClientManager.client = None

    @mock.patch('django_forest.authentication.oidc.client_manager.retrieve', return_value=mocked_config)
    @mock.patch('django_forest.authentication.oidc.client_manager.register', return_value=mocked_client_credentials)
    def test_client_manager(self, mocked_register, mocked_retrieve):
        client = OidcClientManager.get_client_for_callback_url(self.callback_url)
        self.assertTrue(mocked_retrieve.called)
        self.assertTrue(mocked_register.called)
        self.assertTrue(isinstance(client, Client))
        self.assertEqual(client.client_id, 'eyJraWQiOiI2eWthczh5SVMxdHM3dmZZY3JJRTd3aG1Wc2hwbHVKLUlDbU1pRWszYWJ3IiwiYWxnIjoiUlMyNTYifQ.eyJ0b2tlbl9lbmRwb2ludF9hdXRoX21ldGhvZCI6Im5vbmUiLCJyZWRpcmVjdF91cmlzIjpbImh0dHA6Ly9sb2NhbGhvc3Q6ODAwMC9mb3Jlc3QvYXV0aGVudGljYXRpb24vY2FsbGJhY2siXSwicmVnaXN0cmF0aW9uX2VuZHBvaW50IjoiaHR0cHM6Ly9hcGkuZGV2ZWxvcG1lbnQuZm9yZXN0YWRtaW4uY29tL29pZGMvcmVnIiwiYXBwbGljYXRpb25fdHlwZSI6IndlYiIsImdyYW50X3R5cGVzIjpbImF1dGhvcml6YXRpb25fY29kZSJdLCJyZXNwb25zZV90eXBlcyI6WyJjb2RlIl0sImVudmlyb25tZW50X2lkIjoxMDEsImlzcyI6IkZPUkVTVF9BVVRIRU5USUNBVElPTl9TWVNURU0iLCJpYXQiOjE2MjM0Mjc5NTV9.P_mC8eCr36RqRye8YI5zUSGuq4GmpFf8RLvzQmnVMk2HXqzQToPao_RAyvGpYC4JUjFmuTdGI6SSg9JVGIzNod2AXFAxp8qmK8ax_TBKUlg_F_mzrzOxEOaiFkiCQAhavlBhNCx56_CRBmyt_5Uv6L-E4_ezG0A1kacPfOPZaBNSj8FqCRxrsnVZVq60RmY7tRl0DX50ZUuSEvh0zzZLwayBuTYgxLLdg6EvR52dsJHlv0KgNlnHJwKB74cSAl9CM3PVHym9Eg3V3B_BaAbu0eIKmqtJ7_1h1w3-YsH4jAC-czQNgzgO79IiY2s0tTREe4vv-P-__BJ8lIyFxcgzuQ')
        self.assertEqual(client.redirect_uris, [])
        self.assertEqual(client.issuer, 'https://api.development.forestadmin.com')
        self.assertEqual(client.authorization_endpoint, 'https://api.development.forestadmin.com/oidc/auth')
        self.assertEqual(client.token_endpoint, 'https://api.development.forestadmin.com/oidc/token')
        self.assertEqual(client.keyjar.issuer_keys['https://api.development.forestadmin.com'][0].source, 'https://api.development.forestadmin.com/oidc/jwks')

    @mock.patch('django_forest.authentication.oidc.client_manager.retrieve')
    @mock.patch('django_forest.authentication.oidc.client_manager.register')
    def test_client_manager_memo(self, mocked_register, mocked_retrieve):
        OidcClientManager.client = 'foo'
        client = OidcClientManager.get_client_for_callback_url(self.callback_url)
        self.assertEqual(client, 'foo')
        self.assertFalse(mocked_retrieve.called)
        self.assertFalse(mocked_register.called)

    @mock.patch('django_forest.authentication.oidc.client_manager.retrieve', side_effect=Exception('foo'))
    def test_client_manager_exception(self, mocked_retrieve):
        with self.assertRaises(Exception) as cm:
            OidcClientManager.get_client_for_callback_url(self.callback_url)
        self.assertEqual(cm.exception.args[0], 'foo')



# Generated by Django 3.2.4 on 2021-07-01 15:08

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0002_auto_20210615_0942'),
    ]

    operations = [
        migrations.AlterField(
            model_name='choice',
            name='question',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='tests.question'),
        ),
        migrations.AlterField(
            model_name='question',
            name='question_text',
            field=models.CharField(default='how are you?', max_length=200, validators=[django.core.validators.MinLengthValidator(1)]),
        ),
    ]



import json

from django.http import JsonResponse
from django.views.generic import View

from django_forest.authentication.oidc.client_manager import OidcClientManager
from django_forest.authentication.utils import get_callback_url
from django_forest.utils.error_handler import MESSAGES


# Based on https://pyoidc.readthedocs.io/en/latest/examples/rp.html
class IndexView(View):
    def _get_and_check_rendering_id(self, request):
        body_unicode = request.body.decode('utf-8')
        body = json.loads(body_unicode)
        rendering_id = body.get('renderingId', None)
        if not rendering_id:
            raise Exception(MESSAGES['SERVER_TRANSACTION']['MISSING_RENDERING_ID'])

        if not (isinstance(rendering_id, str) or isinstance(rendering_id, int)):
            raise Exception(MESSAGES['SERVER_TRANSACTION']['INVALID_RENDERING_ID'])

        return int(rendering_id)

    def _get_authorization_url(self, redirect_url, state):
        client = OidcClientManager.get_client_for_callback_url(redirect_url)
        args = {
            'client_id': client.client_id,
            'response_type': 'code',
            'scope': ['openid', 'email', 'profile'],
            'state': json.dumps(state),
            'redirect_uri': redirect_url,
        }
        auth_req = client.construct_AuthorizationRequest(request_args=args)
        authorization_url = auth_req.request(client.authorization_endpoint)

        return authorization_url

    def post(self, request, *args, **kwargs):
        try:
            rendering_id = self._get_and_check_rendering_id(request)
            callback_url = get_callback_url()

            authorization_url = self._get_authorization_url(callback_url, {'renderingId': rendering_id})
        except Exception as error:
            return JsonResponse({'errors': [{'status': 500, 'detail': str(error)}]}, status=500)
        else:
            return JsonResponse({'authorizationUrl': authorization_url})
